program Raku2WebSystem;
{
     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     =                   楽々Webシステム                       =
     =      Copyright (c) 2023 中村 州男(Nakamura Kunio)       =
     =      https://sites.google.com/view/nakamura-kunio       =
     =               kunio.nakamura@gosien.net                 =
     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     2023.9.10 Linux ip対応(非推奨コマンド「ifconfig」に加えて)
               PortNo指定時のAjax動作不具合修正(includeフォルダ内「メニューHEADも併せて更新」)
     2023.9.12 ATTACH DATABASE 動作不具合修正
     2023.9.17 LinuxでSQLite3暗号化利用時 PGAGMA_table_info動作不具合修正
     2023.12.12 LinuxでSQLite3 32bitで動作するフォルダのパス指定
     2024.1.2 Lazarus 3.0で宣言が必要になったLazFileUtilsのuses追加，Raspberry Pi 32/64bit対応のlibsqlcipher.so libsqlite3.soの検索場所指定
     2024.2.26 Androidアプリ対応のHTMLパラメタ「AI2」処理追加，Linux 32/64bit対応のlibsqlcipher.so libsqlite3.soの検索場所指定
     2024.3.19 Androidアプリ対応のHTMLパラメタ「AI2」見出し行処理修正
}
{$mode objfpc}{$H+}
uses
  classes, base64, Process , Regexpr, Graphics, sysutils, fpwebfile, BlowFish, FileUtil, Crt, DateUtils,
  fphttpapp, httpdefs, httproute,
  db, sqldb, sqlite3conn,LazFileUtils;

var //Webサーバーの起動から終了まで保持する変数宣言
  Folder,IPAddess:String;    // 本アプリの所在するフォルダ，本マシンのローカルIPアドレス
  SQLiteCipher,SQLite3:String;       // Windows版のsqlcipher.dllまたはSQLite3.dll，Linux版のlibsqlcipher.soまたはlibsqlite3.soのパス+ファイル名
  CipherMode:Boolean;        // データベースの暗号化可否(上記でsqlcipher.dllまたはlibsqlcipher.soがセットされていれば可，さもなくば不可)
  log,Memory:TStringList;    // 本アプリの所在するフォルダ上の「log.txt」ファイルへのログ追加，SELECT文取り出し1件目のデータ等を記憶し『項目名(フィールド名)』で値を参照できる
  Conn: TSQLite3Connection;  // SQLiteデータベース「SQLetc.db」用Connection
  Query: TSQLQuery;          // SQLiteデータベース「SQLetc.db」用クエリー
  SQLname,SQLbun:String;     // SQLパラメタ名，実行中のSQL１文(SQL構文エラー時にブラウザ上に表示するため)

function GetIpAddrList(): string;  // 本マシンのローカルIPアドレスのリスト取得関数
var
    AProcess: TProcess;
    s: string;
    sl: TStringList;
    i, n: integer;

begin
    GetIpAddrList:='';
    sl:=TStringList.Create();
    {$IFDEF WINDOWS}
    AProcess:=TProcess.Create(nil);
    AProcess.CommandLine := 'ipconfig.exe';
    AProcess.Options := AProcess.Options + [poUsePipes, poNoConsole];
    try
      AProcess.Execute();
      Sleep(500); // poWaitOnExit don't work as expected
      sl.LoadFromStream(AProcess.Output);
    finally
      AProcess.Free();
    end;
    for i:=0 to sl.Count-1 do
    begin
      if (Pos('IPv4', sl[i])=0) and (Pos('IP-', sl[i])=0) and (Pos('IP Address', sl[i])=0) then Continue;
      s:=sl[i];
      s:=Trim(Copy(s, Pos(':', s)+1, 999));
      if Pos(':', s)>0 then Continue; // IPv6
      GetIpAddrList:=GetIpAddrList+s+'  ';
    end;
    {$ENDIF}
    {$IFDEF UNIX}

    AProcess:=TProcess.Create(nil);
    AProcess.CommandLine := '/sbin/ip address';
    AProcess.Options := AProcess.Options + [poUsePipes, poWaitOnExit];
    try
      AProcess.Execute();
     sl.LoadFromStream(AProcess.Output);
    finally
      AProcess.Free();
    end;

    for i:=0 to sl.Count-1 do
    begin
      n:=Pos('inet ', sl[i]);
      if n=0 then Continue;
      s:=sl[i];
      s:=Copy(s, n+Length('inet '), 999);
      GetIpAddrList:=GetIpAddrList+Trim(Copy(s, 1, Pos('/', s)-1))+'  ';
    end;

//log.Add(GetIpAddrList);
//log.SaveToFile('log.txt');

    if GetIpAddrList='' then
    begin
      AProcess:=TProcess.Create(nil);
      AProcess.CommandLine := '/sbin/ifconfig';
      AProcess.Options := AProcess.Options + [poUsePipes, poWaitOnExit];
      try
        AProcess.Execute();
       sl.LoadFromStream(AProcess.Output);
      finally
        AProcess.Free();
      end;

      for i:=0 to sl.Count-1 do
      begin
        n:=Pos('inet ', sl[i]);
        if n=0 then Continue;
        s:=sl[i];
        s:=Copy(s, n+Length('inet '), 999);
        GetIpAddrList:=GetIpAddrList+Trim(Copy(s, 1, Pos(' ', s)))+'  ';
      end;
    end;
    {$ENDIF}
    GetIpAddrList:=Trim(StringReplace(GetIpAddrList,'127.0.0.1','',[]));
    sl.Free();
end;

procedure DBlockCheck(DBname:String);  // SQLパラメタの実行等で異常終了した際に，Webサーバーを再起動しなくて済むようにデータベースのロックを解除する処理
var
  CNT:Integer;
  DMY:String;
begin
  CNT:=0;
  while FileExists(DBName+'-journal') do
  begin
    Delay(1000);
    CNT:=CNT+1;
    if CNT>30 then
    begin
{$IFDEF WINDOWS}
      RunCommand('rename '+DBName+' '+DBName+'_bkp',DMY);
      RunCommand('copy '+DBName+'_bkp '+DBName,DMY);
{$ELSE}
      RunCommand('mv '+DBName+' '+DBName+'_bkp',DMY);
      RunCommand('cp '+DBName+'_bkp '+DBName,DMY);
{$ENDIF}
      Break;
    end;
  end;
end;

function GetFilePath(Path,Search:String;Desc:Boolean):String;  // 指定されたパス内(下位階層含む)で，ワイルドカード含む文字列の一致したファイルをファイル名の昇順または降順でリスト取得する関数
var
  PascalFiles: TStringList;
begin
  PascalFiles := TStringList.Create;
  try
    FindAllFiles(PascalFiles, Path, Search, true); //'*.pas;*.pp;*.p;*.inc' find e.g. all pascal sourcefiles
  finally
    if PascalFiles.Count=0 then GetFilePath:=''
    else begin
      PascalFiles.Sort;
      if Desc=false then GetFilePath:=PascalFiles[0]
                    else GetFilePath:=PascalFiles[PascalFiles.Count-1];
    end;
    PascalFiles.Free;
  end;
end;

{$IFDEF WINDOWS}
function GeneratePhrase: string;  // Web管理者や利用者のパスワードを暗号化するためのフレーズ作成関数(Windows版)
var reponse : string;
    resList: TStringList;
    i: integer;
begin
  GeneratePhrase:='';
  RunCommand('getmac /FO CSV /NH',reponse);
{
  log.Add(reponse);
  log.SaveToFile(Folder+'log.txt');
}
  resList:=TStringList.Create;
  resList.Text:=UpperCase(reponse);
  if resList.Count>0 then
  for i:=0 to resList.Count-1 do
    if Pos('TCPIP',resList[i])>0 then
    begin
      GeneratePhrase:=StringReplace(copy(resList[i],2,17),'-','',[rfReplaceAll]);
      break;
    end;
{
    log.Add(GeneratePhrase);
    log.SaveToFile(Folder+'log.txt');
}
end;
{$ELSE}
function GeneratePhrase:string;  // Web管理者や利用者のパスワードを暗号化するためのフレーズ作成関数(Linux版)
var reponse : string;
    re: TRegExpr;
    i: integer;
    MacAddress:String;
begin
  MacAddress:='';
  RunCommand('/sbin/ip',['address','show','dev','eth0'],reponse);
  try
    re := TRegExpr.Create;
//    re.Expression := 'HWaddr ([^ ]+).*inet adr:([^ ]+)';
    re.Expression := '.*ether ([^ ]+)';
    re.Exec(reponse);
    for i:=1 to re.SubExprMatchCount do
//    writeln(re.Match[i]);
      MacAddress:=MacAddress+re.Match[i];
  finally
    re.Free;
  end;
//  log.Add(MacAddress);
//  log.SaveToFile(Folder+'log.txt');
  if MacAddress ='' then
  begin
  RunCommand('/sbin/ip',['address','show','dev','wlan0'],reponse);
  try
    re := TRegExpr.Create;
    re.Expression := '.*ether ([^ ]+)';
    re.Exec(reponse);
    for i:=1 to re.SubExprMatchCount do
//    writeln(re.Match[i]);
      MacAddress:=MacAddress+re.Match[i];
  finally
    re.Free;
  end;
  end;
  if MacAddress ='' then
  begin
  RunCommand('/sbin/ifconfig',['eth0'],reponse);
  try
    re := TRegExpr.Create;
//    re.Expression := 'HWaddr ([^ ]+).*inet adr:([^ ]+)';
    re.Expression := '.*ether ([^ ]+)';
    re.Exec(reponse);
    for i:=1 to re.SubExprMatchCount do
//    writeln(re.Match[i]);
      MacAddress:=MacAddress+re.Match[i];
  finally
    re.Free;
  end;
  end;
  if MacAddress ='' then
  begin
  RunCommand('/sbin/ifconfig',['wlan0'],reponse);
  try
    re := TRegExpr.Create;
    re.Expression := '.*ether ([^ ]+)';
    re.Exec(reponse);
    for i:=1 to re.SubExprMatchCount do
//    writeln(re.Match[i]);
      MacAddress:=MacAddress+re.Match[i];
  finally
    re.Free;
  end;
  end;
  GeneratePhrase:= MacAddress;
end;
{$ENDIF}

function EncryptString(aString,Key:string):string;  // 文字列を復号化する関数
var
    EncrytpStream:TBlowFishEncryptStream;
    StringStream:TStringStream;
    EncryptedString:string;
begin
  StringStream := TStringStream.Create('');
  try
    EncrytpStream := TBlowFishEncryptStream.Create(Key,StringStream);
    EncrytpStream.WriteAnsiString(aString);
    EncrytpStream.Free;
    EncryptedString := StringStream.DataString;
  except
    EncryptedString:='Error';
  end;
  StringStream.Free;
  EncryptString := EncryptedString;
end;

function DecryptString(aString,Key:string):string;  // 文字列を暗号化する関数
var
    DecrytpStream:TBlowFishDeCryptStream;
    StringStream:TStringStream;
    DecryptedString:string;
begin
  StringStream := TStringStream.Create(aString);
  try
    DecrytpStream := TBlowFishDeCryptStream.Create(Key,StringStream);
    DecryptedString := DecrytpStream.ReadAnsiString;
    DecrytpStream.Free;
  except
    DecryptedString:='Error';
  end;
  StringStream.Free;
  DecryptString := DecryptedString;
end;

function BasicAuthorizationView(AResponse: TResponse):TResponse;  // Basic認証窓を表示する関数
begin
  AResponse.Code := 401;
  AResponse.CustomHeaders.Values['WWW-Authenticate'] := 'Basic realm="SECRET AREA"';
  AResponse.CustomHeaders.Values['Connection'] := 'close';
  AResponse.CustomHeaders.Values['Transfer-Encoding'] := 'chunked';
  BasicAuthorizationView:=AResponse;
end;

function GetPassword:String;  // Web管理者が初期設定したパスワードを取得する関数
begin
  GetPassword:='';
    Conn := TSQLite3Connection.Create(nil);
    with Conn do begin
    //    ConnectorType := 'SQLite3';
      SQLiteLibraryName:=SQLiteCipher;// '/usr/lib/i386-linux-gnu/libsqlcipher.so.0.8.6'{'/usr/lib/i386-linux-gnu/libsqlite3.so.0.8.6'};
      DatabaseName := Folder+'SQLetc.db'; {/home/nocode/web/lib/i386-linux}
      open;
      Connected := True;
      Transaction := TSQLTransaction.Create(Conn);
    end;

    Query := TSQLQuery.Create(nil);
    Query.DataBase := Conn;

    Query.SQL.Text := 'SELECT * FROM 利用者情報 WHERE 所属 == "Web管理者" LIMIT 1';
    Query.Open;
    if not Query.EOF then
    begin
      GetPassword:=DecryptString(DecodeStringBase64(Query.FieldByName('暗号化パスワード').AsString),GeneratePhrase);
    end;
    Query.Close;
    Query.Free;
    Conn.Free;
end;

function GetBasicAuth(LAuth,UserOrPass: String):String;  // Basic認証窓に入力され引き継いでいるユーザー名・パスワードを取得する関数
var
  User,Pass:String;
begin
  GetBasicAuth:='';
  if Copy(LAuth,1,5) = 'Basic' then
  begin
    User:=DecodeStringBase64(Copy(LAuth,7,Length(LAuth) - 7 + 1));
    Pass:=copy(User,Pos(':',User)+1,MaxInt);
    User:=copy(User,1,Pos(':',User)-1);
    if UserOrPass<>'Pass' then GetBasicAuth:=User else GetBasicAuth:=Pass;
  end;
end;

function BasicAuthorization(LAuth: String):String;  // Basic認証窓に入力され引き継いでいるユーザー名から所属を取得する関数
var
  User,Pass:String;
begin
  BasicAuthorization:='null';
  if Copy(LAuth,1,5) = 'Basic' then
  begin
    User:=GetBasicAuth(LAuth,'User');
    Pass:=EncryptString(GetBasicAuth(LAuth,'Pass'),GeneratePhrase);
    Conn := TSQLite3Connection.Create(nil);
    with Conn do begin
    //    ConnectorType := 'SQLite3';
      SQLiteLibraryName:=SQLiteCipher;// '/usr/lib/i386-linux-gnu/libsqlcipher.so.0.8.6'{'/usr/lib/i386-linux-gnu/libsqlite3.so.0.8.6'};
      DatabaseName := Folder+'SQLetc.db'; {/home/nocode/web/lib/i386-linux}
      open;
      Connected := True;
      Transaction := TSQLTransaction.Create(Conn);
    end;

    Query := TSQLQuery.Create(nil);
    Query.DataBase := Conn;

    Query.SQL.Text := 'SELECT * FROM 利用者情報 WHERE 名前 == :name';
    Query.Params.ParamByName('name').AsString := User;
    Query.Open;
    if not Query.EOF then
    begin
      if EncodeStringBase64(Pass) = Query.FieldByName('暗号化パスワード').AsString
        then BasicAuthorization:=Query.FieldByName('所属').AsString;
    end;
    Query.Close;
    Query.Free;
    Conn.Free;
  end;
end;

///////////////////////////////////////////////////////////////////////////////////////////////////
//  「http://127.0.0.1(またはローカルIPアドレス)/?SQL=SQLパラメタ&HTML=HTMLパラメタ」CGI実行部   //
///////////////////////////////////////////////////////////////////////////////////////////////////
procedure CGI(ARequest: TRequest; AResponse: TResponse);
var //CGIのリクエストからレスポンスまで保持するメイン処理・関数・サブ処理内で共通の変数宣言
  Conn: TSQLite3Connection;    // SQLiteデータベース「SQLパラメタ内で指定されたDB名等」用Connection
  Query: TSQLQuery;            // SQLiteデータベース「SQLパラメタ内で指定されたDB名等」用クエリー
  MenuNameList,MenuDataList,NameList1,DataList1,NameList2,DataList2,NameList3,DataList3,NameList4,DataList4,SQL,etc:TStringList; // HTMLパラメタ内置換文字「＠メニュー＠，＠データ＠，＠データ2＠，＠データ3＠，＠データ4＠」用データリスト
  SW:Integer;                  // SELECT文で取り出したレコードを上行のNameList1～4,DataList1～4のいずれにセットするかの1～4の値(SQL制御行にクリアがあると，上行のデータ1～4をクリアし，SW:=0に戻す)
  TABset,SQLexecFirst:Boolean; // SQLパラメタ名に「tabset」が含まれる場合に「入力一件分のSQL文等」にある最初のSELECT文の取り出しレコードをTAB列にセットするか否か，CGIのリクエスト最初1回だけSQLパラメタ内文字置換処理をするための初回か否か

function GetSQLetc(SQLorHTMLorMENU,Namae,Kubun,org:String):TStringList; // SQLetc.db内のSQLパラメタ(区分別)・HTMLパラメタ・メニュー(所属別)取出し関数
var
  Field,S:String;
  i:Integer;
begin
  GetSQLetc:=TStringList.Create;  GetSQLetc.Clear;
  if (SQLorHTMLorMENU = 'SQL') or (SQLorHTMLorMENU = 'HTML') then
  begin
    Conn := TSQLite3Connection.Create(nil);
    with Conn do begin
    //    ConnectorType := 'SQLite3';
      SQLiteLibraryName:=SQLiteCipher;// '/usr/lib/i386-linux-gnu/libsqlcipher.so.0.8.6'{'/usr/lib/i386-linux-gnu/libsqlite3.so.0.8.6'};
      DatabaseName := Folder+'SQLetc.db'; {/home/nocode/web/lib/i386-linux}
      open;
      Connected := True;
      Transaction := TSQLTransaction.Create(Conn);
    end;

    Query := TSQLQuery.Create(nil);
    Query.DataBase := Conn;
    Query.SQL.Text :='SELECT * FROM '+SQLorHTMLorMENU+' WHERE 名称 == '''+Namae+''';';
    try
      Query.Open;
    except
    end;
    if Query.RecordCount>0 then
    begin
      if SQLorHTMLorMENU = 'SQL' then
      begin
        if (Query.FieldByName('実行権限').AsString='') or (Pos(org,Query.FieldByName('実行権限').AsString)>0) then
        if trim(Query.FieldByName(Kubun).AsString)<>'' then
        begin
          S:=Query.FieldByName('データベース').AsString;
          if Pos('./',S)=1 then S:=Folder+copy(S,3,MaxInt);
          {$IFDEF WINDOWS}
          S:=StringReplace(S,'/','\',[rfReplaceAll]);
          {$ENDIF}
          GetSQLetc.Add(S);
          GetSQLetc.Add(Query.FieldByName('暗号化').AsString);
          GetSQLetc.Add(Query.FieldByName(Kubun).AsString);
        end;
      end
      else GetSQLetc.Add(Query.FieldByName('内容').AsString);
    end;
    Query.Close;
    Query.Free;
    Conn.Free;
  end
  else if (SQLorHTMLorMENU = 'MENU') then
  begin
    Conn := TSQLite3Connection.Create(nil);
    with Conn do begin
    //    ConnectorType := 'SQLite3';
      SQLiteLibraryName:=SQLiteCipher;// '/usr/lib/i386-linux-gnu/libsqlcipher.so.0.8.6'{'/usr/lib/i386-linux-gnu/libsqlite3.so.0.8.6'};
      DatabaseName := Folder+'SQLetc.db'; {/home/nocode/web/lib/i386-linux}
      open;
      Connected := True;
      Transaction := TSQLTransaction.Create(Conn);
    end;

    Query := TSQLQuery.Create(nil);
    Query.DataBase := Conn;
    Query.SQL.Text :='SELECT * FROM メニュー WHERE 所属 == '''+org+''' ORDER BY 親 ASC, 子 ASC, 孫 ASC;';
    try
      Query.Open;
    except
    end;
    if Query.RecordCount>0 then
    begin
      if SQLorHTMLorMENU = 'MENU' then
      begin
        Query.GetFieldNames(MenuNameList);
        while not Query.EOF do
        begin
          S:='';
          if Query.Fields.Count>0 then
            for i:=0 to Query.Fields.Count-1 do
            begin
              Field:=Query.Fields[i].AsString;
              S:=S+Field+#9;
            end;
            MenuDataList.Add(S);
            Query.Next;
        end;
      end;
    end;
    Query.Close;
    Query.Free;
    Conn.Free;
  end;
end;

procedure DateMemoryValueSet(SQLtext:String); // SQLパラメタ内の『今日+n』に対する値をSELECT文の1件目等の参照文字リストMemoryにセットする処理
var
  MOJI:String;
  S:WideString;
begin
  S:=SQLtext;
  while Pos('『今日+',S)>0 do
  begin
    MOJI:=Copy(S,Pos('『今日+',S)+4,MaxInt);
    if Pos('』',MOJI)<=0 then Break;
    MOJI:=Copy(MOJI,1,Pos('』',MOJI)-1);
//log.Add('『'+MOJI+'』');
//log.SaveToFile(Folder+'log.txt');
    if Memory.Values['今日+'+MOJI]='' then Memory.Values['今日+'+MOJI]:=FormatDateTime('yyyy-mm-dd',IncDay(now,StrToIntDef(MOJI,0)));
    try
      S:=StringReplace(S,'『今日+'+MOJI+'』','',[rfIgnoreCase,rfReplaceAll]);
    except
    end;
  end;
end;

function Inset(SQLIn,LPNo:String):String; // SQL1文内の置換文字「」『』に対する値セット・行列データセット・複数選択ボックス・日付形式変換の関数
var
  MOJI,RepMOJI,Col:String;
  S,S1,S2:WideString;
  S3,S4:WideString;
  Cols:TStringList;
  i,ColNo:Integer;
begin
//  log.Add('SQLIn:'+S+'   LPNo:'+LPNo);
//  log.SaveToFile(Folder+'log.txt');
  S:=SQLIn;
  if SQLexecFirst then
  begin
    SQLexecFirst:=False;
    if ARequest.ContentFields.Values['SQL']='SQL・HTML作成' then
    begin
      for i:=0 to ARequest.ContentFields.Count-1 do
       if ARequest.ContentFields.ValueFromIndex[i]<>'' then
        ARequest.ContentFields.ValueFromIndex[i]:=StringReplace(StringReplace(StringReplace(StringReplace(ARequest.ContentFields.ValueFromIndex[i],'「','〔',[rfIgnoreCase,rfReplaceAll]),'」','〕',[rfIgnoreCase,rfReplaceAll]),'『','｛',[rfIgnoreCase,rfReplaceAll]),'』','｝',[rfIgnoreCase,rfReplaceAll]);
    end;
    if Pos('複数選択',ARequest.ContentFields.Text)>0 then
    begin
      for i:=0 to ARequest.ContentFields.Count-1 do
       if Pos('複数選択',ARequest.ContentFields.Names[i])>0 then
       begin
         RepMOJI:=StringReplace(ARequest.ContentFields.Names[i],'複数選択','',[]);
         if ARequest.ContentFields.Values[RepMOJI]=''
           then ARequest.ContentFields.Values[RepMOJI]:=ARequest.ContentFields.ValueFromIndex[i]
           else ARequest.ContentFields.Values[RepMOJI]:=ARequest.ContentFields.Values[RepMOJI]+','+ARequest.ContentFields.ValueFromIndex[i];
       end;
    end;
  end;
  while Pos('『',S)>0 do
  begin
    MOJI:=Copy(S,Pos('『',S)+1,MaxInt);
    if Pos('』',MOJI)<=0 then Break;
    MOJI:=Copy(MOJI,1,Pos('』',MOJI)-1);
//log.Add('『'+MOJI+'』');
//log.SaveToFile(Folder+'log.txt');
    try
      S:=StringReplace(S,'『'+MOJI+'』',Memory.Values[MOJI],[rfIgnoreCase,rfReplaceAll]);
    except
    end;
  end;
  if Pos(#9,LPNo)>0 then
  begin
//log.Add('列:'+LPNo);
//log.SaveToFile(Folder+'log.txt');
    Col:=LPNo;
    Cols:=TStringList.Create;
    while Pos(#9,Col)>0 do
    begin
      Cols.Add(copy(Col,1,Pos(#9,Col)-1));
      Col:=copy(Col,Pos(#9,Col)+1,MaxInt);
    end;
    while Pos('「列:',S)>0 do
    begin
      MOJI:=Copy(S,Pos('「列:',S)+3,MaxInt);
      if Pos('」',MOJI)<=0 then Break;
      MOJI:=Copy(MOJI,1,Pos('」',MOJI)-1);
      ColNo:=StrToIntDef(MOJI,-1);
      if (ColNo>0) and (ColNo<=Cols.Count) then
      begin
//log.Add('列番号:'+MOJI+'='+Cols[ColNo-1]);
//log.SaveToFile(Folder+'log.txt');
        S:=StringReplace(S,'「列:'+MOJI+'」',Cols[ColNo-1],[rfIgnoreCase,rfReplaceAll]);
        Memory.Values['前列:'+MOJI]:=Cols[ColNo-1];
      end
      else S:=StringReplace(S,'「列:'+MOJI+'」','',[rfIgnoreCase,rfReplaceAll]);
    end;
    Cols.Free;
  end;

  while Pos('「',S)>0 do
  begin
    MOJI:=Copy(S,Pos('「',S)+1,MaxInt);
    if Pos('」',MOJI)<=0 then Break;
    MOJI:=Copy(MOJI,1,Pos('」',MOJI)-1);
    if (Pos('暗号化',MOJI)>0) then
    begin
      RepMOJI:=StringReplace(MOJI+LPNo,'暗号化','',[]);
      if (ARequest.ContentFields.Values[RepMOJI]<>'') then
      begin
        RepMOJI:= ARequest.ContentFields.Values[RepMOJI];
        ARequest.ContentFields.Values[MOJI+LPNo]:= EncodeStringBase64(EncryptString(RepMOJI,GeneratePhrase));
      end;
    end;
//log.Add(LPNo+MOJI+':'+ARequest.ContentFields.Values[LPNo+MOJI]);
//log.SaveToFile(Folder+'log.txt');
    if ARequest.ContentFields.Values[MOJI+LPNo]<>''
      then  S:=StringReplace(S,'「'+MOJI+'」',StringReplace(ARequest.ContentFields.Values[MOJI+LPNo],'''','''''',[rfIgnoreCase,rfReplaceAll]),[rfIgnoreCase,rfReplaceAll])
      else if ARequest.ContentFields.Values[MOJI]<>''
             then S:=StringReplace(S,'「'+MOJI+'」',StringReplace(ARequest.ContentFields.Values[MOJI],'''','''''',[rfIgnoreCase,rfReplaceAll]),[rfIgnoreCase,rfReplaceAll])
             else S:=StringReplace(S,'「'+MOJI+'」',StringReplace(ARequest.QueryFields.Values[MOJI],'''','''''',[rfIgnoreCase,rfReplaceAll]),[rfIgnoreCase,rfReplaceAll]) ;
    if LPNo<>'' then Memory.Values['前:'+MOJI]:=ARequest.ContentFields.Values[MOJI+LPNo];
  end;
  if ARequest.ContentFields.Values['SQL']='SQL・HTML作成' then
  begin
    S:=StringReplace(StringReplace(StringReplace(StringReplace(S,'〔','「',[rfIgnoreCase,rfReplaceAll]),'〕','」',[rfIgnoreCase,rfReplaceAll]),'｛','『',[rfIgnoreCase,rfReplaceAll]),'｝','』',[rfIgnoreCase,rfReplaceAll]);
  end;
  while Pos('日付形式変換（',S)>0 do
  begin
    S1:=copy(S,Pos('日付形式変換（',S),MaxInt);
    if Pos('）',S1)>0 then
    begin
      S1:=copy(S1,1,Pos('）',S1));
      S2:=StringReplace(StringReplace(S1,'日付形式変換（','',[]),'）','',[]);
      if length(StringReplace(StringReplace(S2,'/','',[rfReplaceAll]),'-','',[rfReplaceAll]))=8 then S2:=copy(S2,1,4)+'-'+copy(S2,5,2)+'-'+copy(S2,7,2)
      else if (Pos('/',S2)>0) or (Pos('-',S2)>0) then
      begin
        if length(S2)=10 then
        begin
          if Pos('/',S2)>0 then S2:=StringReplace(S2,'/','-',[rfReplaceAll]);
        end else
        begin
          S2:=StringReplace(S2,'/','-',[rfReplaceAll]);
          S3:=copy(S2,1,Pos('-',S2));
          if length(S3)=5 then S4:=S3 else if length(S3)=3 then S4:='20'+S3 else if length(S3)=2 then S4:='200'+S3 else S4:=S3;
          S2:=copy(S2,Pos('-',S2)+1,MaxInt);
          S3:=copy(S2,1,Pos('-',S2));
          if length(S3)=3 then S4:=S4+S3 else if length(S3)=2 then S4:=S4+'0'+S3 else S4:=S4+S3;
          S3:=copy(S2,Pos('-',S2)+1,MaxInt);
          if length(S3)=2 then S4:=S4+S3 else if length(S3)=1 then S4:=S4+'0'+S3 else S4:=S4+S3;
          S2:=S4;
        end;
      end;
      S:=StringReplace(S,S1,S2,[]);
    end else Break;
  end;
  Inset:=StringReplace(StringReplace(StringReplace(S,'￥；',';',[rfIgnoreCase,rfReplaceAll]),#13,'',[rfIgnoreCase,rfReplaceAll]),#10,''' || char(10) || ''',[rfIgnoreCase,rfReplaceAll]);
end;

function InsetHTML(HTMLIn:String):String; // HTMLパラメタ内のincludeフォルダ内差込み文字【】・置換文字「」『』に対する値セット
var
  MOJI,MOJI2:String;
  S:WideString;
  HTML:TStringList;
begin
  S:=HTMLIn;
  while Pos('【',S)>0 do
  begin
    MOJI:=Copy(S,Pos('【',S)+1,MaxInt);
    if Pos('】',MOJI)<=0 then Break;
    MOJI:=Copy(MOJI,1,Pos('】',MOJI)-1);
//log.Add(MOJI);
//log.SaveToFile(Folder+'log.txt');
    if FileExists(Folder+'include/'+MOJI) then
    begin
      HTML:=TStringList.Create;
      HTML.LoadFromFile(Folder+'include/'+MOJI);
      S:=StringReplace(S,'【'+MOJI+'】',HTML.Text,[rfIgnoreCase,rfReplaceAll]);
      HTML.Free;
    end
    else S:=StringReplace(S,'【'+MOJI+'】','',[rfIgnoreCase,rfReplaceAll]);
  end;
  while Pos('「',S)>0 do
  begin
    MOJI:=Copy(S,Pos('「',S)+1,MaxInt);
    if Pos('」',MOJI)<=0 then Break;
    MOJI:=Copy(MOJI,1,Pos('」',MOJI)-1);
//log.Add(MOJI+':'+ARequest.ContentFields.Values[MOJI]);
//log.SaveToFile(Folder+'log.txt');
    if ARequest.ContentFields.Values[MOJI]<>''
      then S:=StringReplace(S,'「'+MOJI+'」',ARequest.ContentFields.Values[MOJI],[rfIgnoreCase,rfReplaceAll])
      else S:=StringReplace(S,'「'+MOJI+'」',ARequest.QueryFields.Values[MOJI],[rfIgnoreCase,rfReplaceAll]) ;
  end;
  while Pos('『',S)>0 do
  begin
    MOJI:=Copy(S,Pos('『',S)+1,MaxInt);
    if Pos('』',MOJI)<=0 then Break;
    MOJI:=Copy(MOJI,1,Pos('』',MOJI)-1);
//log.Add('『'+MOJI+'』');
//log.SaveToFile(Folder+'log.txt');
    MOJI2:=Memory.Values[MOJI];
    if (ARequest.QueryFields.Values['SQL']='SQL編集') or (ARequest.QueryFields.Values['SQL']='HTML編集') then
        MOJI2:=StringReplace(StringReplace(StringReplace(MOJI2,'『','｛',[rfIgnoreCase,rfReplaceAll]),'』','｝',[rfIgnoreCase,rfReplaceAll]),'</textarea>','&lt;/textarea>',[rfIgnoreCase,rfReplaceAll]);
    try
      S:=StringReplace(S,'『'+MOJI+'』',MOJI2,[rfIgnoreCase,rfReplaceAll]);
    except
    end;
  end;
  if (ARequest.QueryFields.Values['SQL']='SQL編集') or (ARequest.QueryFields.Values['SQL']='HTML編集') then
      S:=StringReplace(StringReplace(S,'｛','『',[rfIgnoreCase,rfReplaceAll]),'｝','』',[rfIgnoreCase,rfReplaceAll]);
  InsetHTML:=S;
end;

procedure SQLexecute(LPNo:String;First,Last,View:Boolean); // SQLパラメタの区分単位に複数のSQLと分岐制御・クリア制御の実行処理
var
  i,Line:Integer;
  Field,S,JumpLabel,Left,Right,Cond:String;
  SQLexec,NameCHKList:TStringList;
  QueryFirst:Boolean;
  WideS:WideString;
begin
  NameCHKList:=TStringList.Create;
  SQLexec:=TStringList.Create;
  SQLexec.Text:=SQL.Text;
  if First then
  begin
    DBlockCheck(SQLexec[0]);
    Conn := TSQLite3Connection.Create(nil);
    with Conn do begin
//      ConnectorType := 'SQLite3';
      SQLiteLibraryName:=SQLiteCipher;// '/usr/lib/i386-linux-gnu/libsqlcipher.so.0.8.6'{'/usr/lib/i386-linux-gnu/libsqlite3.so.0.8.6'};
      if SQLite3<>'' then if Pos('pragma_table_info',LowerCase(SQLexec.Text))>=0 then SQLiteLibraryName:=SQLite3;
      DatabaseName := SQLexec[0]; {/home/nocode/web/lib/i386-linux}
      if CipherMode then if SQLexec[1]='有' then Password := GetPassword; // The current password
      try
        open;
      except
        DBlockCheck(SQLexec[0]);
        open;
      end;
      Transaction := TSQLTransaction.Create(Conn);
      Transaction.Active := False;
      if SQLexec.Count>2 then if (UpperCase(SQLexec[2])='VACUUM;') {or (Pos('ATTACH',UpperCase(SQLexec[2]))=1)} then
      begin
        ExecuteDirect('End Transaction');  // End the transaction started by SQLdb
        ExecuteDirect(SQLexec[2]);
        ExecuteDirect('Begin Transaction'); //Start a transaction for SQLdb to use
        SQLexec.Delete(2);
      end;
    end;
  end;
  SQLexec.Delete(0); SQLexec.Delete(0);
  if SQLexec.Count>0 then
  for i:=0 to SQLexec.Count-1 do
  begin
    if Pos('--',SQLexec[i])=1 then SQLexec[i]:=SQLexec[i]+'＃##＃';
    if Pos('ATTACH',UpperCase(SQLexec[i]))=1 then
    begin
      if Pos('./',SQLexec[i])>0 then SQLexec[i]:=StringReplace(SQLexec[i],'./',Folder,[]);
      {$IFDEF WINDOWS}
      SQLexec[i]:=StringReplace(SQLexec[i],'/','\',[rfReplaceAll]);
      {$ENDIF}
      Conn.ExecuteDirect('End Transaction');  // End the transaction started by SQLdb
      Conn.ExecuteDirect(SQLexec[i]);
      Conn.ExecuteDirect('Begin Transaction'); //Start a transaction for SQLdb to use
      SQLexec[i]:='';
    end;
  end;
  SQLbun:=StringReplace(StringReplace(SQLexec.Text,#13,'',[rfIgnoreCase,rfReplaceAll]),#10+'--','＃#＃',[rfIgnoreCase,rfReplaceAll]);
  SQLbun:=StringReplace(StringReplace(StringReplace(SQLbun,#10,' ',[rfIgnoreCase,rfReplaceAll]),'\;','￥；',[rfIgnoreCase,rfReplaceAll]),';',';'+#10,[rfIgnoreCase,rfReplaceAll]);
  SQLbun:=StringReplace(SQLbun,'＃#＃',#10+'--',[rfIgnoreCase,rfReplaceAll]);
  SQLexec.Text:=StringReplace(SQLbun,'＃##＃',#10,[rfIgnoreCase,rfReplaceAll]);
  if First then if Conn.Transaction.Active = false then Conn.Transaction.StartTransaction;
  Line:=0;
  while Line < SQLexec.Count do
  begin
   if trim(SQLexec[Line])<>'' then
   begin
    if Pos('--',SQLexec[Line])<>1 then
    begin
      if First then
      begin
        Query := TSQLQuery.Create(nil);
        Query.DataBase := Conn;
      end;
      Query.SQL.Clear;
//      log.Add(IntToStr(Length(Inset(SQLexec[Line],LPNo)))+' :'+Inset(SQLexec[Line],LPNo));
//      log.SaveToFile(Folder+'log.txt');
      Query.SQL.Text:=Inset(SQLexec[Line],LPNo);
//log.Add('begin:'+Query.SQL.Text);
//log.SaveToFile(Folder+'log.txt');
      if Pos('SELECT',UpperCase(trim(Query.SQL.Text)))=1 then
      begin
        Query.Open;
        if View then
        begin
          Query.GetFieldNames(NameCHKList);
          if SW=3 then if NameList3.Text <> NameCHKList.Text then begin SW:=4; NameList4.Assign(NameCHKList); end;
          if SW=2 then if NameList2.Text <> NameCHKList.Text then begin SW:=3; NameList3.Assign(NameCHKList); end;
          if SW=1 then if NameList1.Text <> NameCHKList.Text then begin SW:=2; NameList2.Assign(NameCHKList); end;
          if SW=0 then begin SW:=1; NameList1.Assign(NameCHKList); end;
        end;
        QueryFirst:=True;
        while not Query.EOF do
        begin
          S:='';
          if Query.Fields.Count>0 then
            for i:=0 to Query.Fields.Count-1 do
            begin
              Field:=Query.Fields[i].AsString;
              if QueryFirst then Memory.Values[Query.Fields[i].FieldName]:=Field;
              if View or TABset then S:=S+Field+#9;
            end;
          if View or TABset then
          begin
            if TABset then ARequest.ContentFields.Values['TAB列']:=ARequest.ContentFields.Values['TAB列']+S+#10 else
            if SW=1 then DataList1.Add(S) else if SW=2 then DataList2.Add(S) else if SW=3 then DataList3.Add(S) else DataList4.Add(S);
            Query.Next;
            QueryFirst:=False;
          end else Break;
        end;
        TABset:=False;
        Query.Close;
      end
      else
      begin
        Query.ExecSQL;
//log.Add('end:'+Query.SQL.Text);
//log.SaveToFile(Folder+'log.txt');
      end;
      if last then
      begin
      end;
    end
    else begin //コメント行の処理
//      log.Add(SQLexec[Line]+' <- '+LPNo);
//      log.SaveToFile(Folder+'log.txt');
      WideS:=Inset(SQLexec[Line],LPNo);
      if Pos('→',WideS)>1 then
      begin
        JumpLabel:=trim(copy(WideS,Pos('→',WideS)+1,MaxInt));
        WideS:=StringReplace(StringReplace(WideS,'(','（',[]),')','）',[]);
        if Pos('（',WideS)>1 then
        begin
          WideS:=copy(WideS,Pos('（',WideS)+1,MaxInt);
          if Pos('）',WideS)>1 then
          begin
            WideS:=copy(WideS,1,Pos('）',WideS)-1);
            Cond:='';
            if Pos('＝',WideS)>0 then Cond:='＝';
            if Pos('≠',WideS)>0 then Cond:='≠';
            if Pos('≦',WideS)>0 then Cond:='≦';
            if Pos('≧',WideS)>0 then Cond:='≧';
            if Pos('＜',WideS)>0 then Cond:='＜';
            if Pos('＞',WideS)>0 then Cond:='＞';
            if Cond='' then JumpLabel:='' else
            begin
              if Pos(Cond,WideS)>1 then Left:=copy(WideS,1,Pos(Cond,WideS)-1) else Left:='';
              if Pos(Cond,WideS)<length(WideS) then Right:=copy(WideS,Pos(Cond,WideS)+1,MaxInt) else Right:='';
              if Cond='＝' then if Left=Right then else JumpLabel:='';
              if Cond='≠' then if Left<>Right then else JumpLabel:='';
              if Cond='≦' then if StrToFloatDef(Left,0)<=StrToFloatDef(Right,0) then else JumpLabel:='';
              if Cond='≧' then if StrToFloatDef(Left,0)>=StrToFloatDef(Right,0) then else JumpLabel:='';
              if Cond='＜' then if StrToFloatDef(Left,0)<StrToFloatDef(Right,0) then else JumpLabel:='';
              if Cond='＞' then if StrToFloatDef(Left,0)>StrToFloatDef(Right,0) then else JumpLabel:='';
            end;
          end
          else JumpLabel:='';
        end;
        if JumpLabel<>'' then
          while Line+1 < SQLexec.Count do
            if SQLexec[Line+1]='--'+JumpLabel then Break else Line+=1;
      end
      else if Pos('クリア',WideS)>1 then begin SW:=0; NameList1.Clear; DataList1.Clear; NameList2.Clear; DataList2.Clear; NameList3.Clear; DataList3.Clear; end;
    end;
   end;
   Line+=1;
  end;
  if Last then
  begin
    try
      if Conn.Transaction.Active then Conn.Transaction.Commit;
      Query.Close;
      Query.Free;
      Conn.Free;
    except
    end;
//Memory.SaveToFile(Folder+'Memory.txt');
  end;
  NameCHKList.Free;
  SQLexec.Free;
end;

function DBtoHTML(NameList,DataList:TStringList):String; // SELECT文で取り出したレコードを見出し行列＋データ行列の2次元配列形式データに変換する関数
var
  S,SubList:String;
  i,j:Integer;
begin
  S:='';
  if NameList.Count>0 then
  if DataList.Count>0 then
  begin
    S:=S+'[';
    for j:=0 to NameList.Count-1 do
    if trim(NameList[j])<>'' then
    begin
//  log.Add(NameList[j]+'='+SubList);
//  log.SaveToFile(Folder+'log.txt');
      S:=S+'"'+NameList[j]+'"';
      if j<NameList.Count-1 then S:=S+',';
    end;
    S:=S+'],';
    for i:=0 to DataList.Count-1 do
    if pos(#9,DataList[i])>0 then
    begin
      SubList:=DataList[i]+#9;
      S:=S+'[';
      for j:=0 to NameList.Count-1 do
      if trim(NameList[j])<>'' then
      begin
//log.Add(NameList[j]+'='+SubList);
//log.SaveToFile(Folder+'log.txt');
        S:=S+'"'+copy(SubList,1,Pos(#9,SubList)-1)+'"';
        if j<NameList.Count-1 then S:=S+',';
        SubList:=copy(SubList,Pos(#9,SubList)+1,MaxInt)+#9;
      end;
      S:=S+']';  if i<DataList.Count-1 then S:=S+',';
    end;
  end;
  DBtoHTML:=S;
end;

function DBtoJSON(NameList,DataList:TStringList):String; // SQLパラメタ名に「json」が含まれる場合に，SELECT文で取り出したレコードを{"見出し:データ",…}のJSON形式データに変換する関数
var
  S,SubList:String;
  i,j:Integer;
begin
  S:='';
  if NameList.Count>0 then
  if DataList.Count>0 then
  begin
    for i:=0 to DataList.Count-1 do
    if pos(#9,DataList[i])>0 then
    begin
      SubList:=DataList[i]+#9#9#9#9#9;
      S:=S+'{';
      for j:=0 to NameList.Count-1 do
      if trim(NameList[j])<>'' then
      begin
//log.Add(NameList[j]+'='+SubList);
//log.SaveToFile(Folder+'log.txt');
        S:=S+'"'+NameList[j]{.Values[IntToStr(j)]}+'":"'+copy(SubList,1,Pos(#9,SubList)-1)+'"';
        if j<NameList.Count-1 then S:=S+',';
        SubList:=copy(SubList,Pos(#9,SubList)+1,MaxInt);
      end;
      S:=S+'}';  if i<DataList.Count-1 then S:=S+',';
    end;
  end;
  DBtoJSON:=S;
end;

//-----------------------------------------------------------------------------------------------------
//--「http://127.0.0.1(またはローカルIPアドレス)/?SQL=SQLパラメタ&HTML=HTMLパラメタ」CGIメイン実行部 --
//-----------------------------------------------------------------------------------------------------
var //CGIのリクエストからレスポンスまで保持するメイン処理用の変数宣言(これより上の関数や処理とは無関係)
  i,j,k,CNT,Li:Integer;       // for文用変数i,j,kと入力フォームのname="行番号:n"の数の合計を保持するCNT
  First,Last,Loop:Boolean; // 行列データの最初，最後のデータか否か(行列データ以外ならば両方ともTrue),行列データ(行番号・CSV列・TAB列・CSVファイル)ならばTrue
  Rows,csv:TStringList;    // TAB形式の行列データ,CSV形式の行列データまたはCSV形式ファイルのデータ保持リスト
  f: TUploadedFile;        // CSV形式ファイルのアップロード用
  OutStream:TStringStream; // CSV形式ファイルのアップロード用
  org,etcName,S:String;      // 所属，HTMLパラメタ名(省略時「一覧表」)
  UploadPath,UploadFileName:String;       // ファイルアップロードのimages下位用パスとファイル名変更用
  UploadFileStream:TFileStream; // ファイルのアップロード用
begin
  SW:=0;
  org:=BasicAuthorization(ARequest.Authorization);
  if org='null' then begin AResponse:= BasicAuthorizationView(AResponse); exit; end;
  MenuNameList:=TStringList.Create;
  MenuDataList:=TStringList.Create;
  SQL:=GetSQLetc('MENU','','',org); //'入力一件分のSQL文等' '表示用のSQL文等'

  if ARequest.QueryFields.Count+ARequest.ContentFields.Count=0 then
  begin
    etc:=GetSQLetc('HTML','メニュー','','');
    if (ARequest.RemoteAddress='127.0.0.1') or ( LowerCase(ARequest.RemoteAddress)='localhost') then
      begin
        if IPAddess='' then IPAddess:=GetIpAddrList;
        AResponse.Content := StringReplace(StringReplace(InsetHTML(etc.Text),'＠メニュー＠',DBtoHTML(MenuNameList,MenuDataList),[]),'＠所属＠',org+'('+IPAddess+')',[rfReplaceAll])
      end
      else AResponse.Content := StringReplace(StringReplace(InsetHTML(etc.Text),'＠メニュー＠',DBtoHTML(MenuNameList,MenuDataList),[]),'＠所属＠',org,[rfReplaceAll]);
   exit;
  end;
  NameList1:=TStringList.Create;
  DataList1:=TStringList.Create;
  NameList2:=TStringList.Create;
  DataList2:=TStringList.Create;
  NameList3:=TStringList.Create;
  DataList3:=TStringList.Create;
  NameList4:=TStringList.Create;
  DataList4:=TStringList.Create;
//  SQL:=TStringList.Create;
//  etc:=TStringList.Create;
//  ARequest.ContentFields.GetNameValue(0,GetName,GetValue);
//  ARequest.QueryFields.GetNameValue(0,GetName,GetValue);

  if UpperCase(ARequest.QueryFields.Values['HTML'])='AI2' then
    if ARequest.ContentFields.Count=0 then ARequest.ContentFields.Assign(ARequest.QueryFields);

  SQLname:=ARequest.ContentFields.Values['SQL'];

log.Add('----START---- '+FormatDateTime('yyyy-mm-dd hh:nn:ss zzz',now)+'ms IPAddress:'+ARequest.RemoteAddress+' User:'+GetBasicAuth(ARequest.Authorization,'User')+'('+org+') SQL:'+SQLname+ARequest.QueryFields.Values['SQL']);
log.SaveToFile(Folder+'log.txt');

  Memory.Clear;
  Memory.Values['今日']:=FormatDateTime('yyyy-mm-dd',now);
  Memory.Values['今日時']:=FormatDateTime('yyyy-mm-dd hh:nn:ss',now);
  Memory.Values['今時']:=FormatDateTime('hh:nn:ss',now);

  SQLexecFirst:=True;
  TABset:=False;
  if Pos('TABSET',UpperCase(ARequest.QueryFields.Values['SQL']))>0 then
  begin
    TABset:=True;
    SQLname:=ARequest.QueryFields.Values['SQL'];
  end;
  if SQLname='' then SQLname:=ARequest.QueryFields.Values['SQL'] else
  begin
    SQL:=GetSQLetc('SQL',SQLname,'入力開始時のSQL文等',org); //'入力一件分のSQL文等' '表示用のSQL文等'
    if SQL.Count>0 then
    begin
      if Pos('『今日+',SQL.text)>0 then DateMemoryValueSet(SQL.text);
      SQLexecute('',True,True,False);
      TABset:=False;
    end;
    SQL:=GetSQLetc('SQL',SQLname,'入力一件分のSQL文等',org); //'入力開始時のSQL文等' '表示用のSQL文等'
//    log.Add('入力一件分のSQL文等');
//    log.SaveToFile(Folder+'log.txt');
    Loop:=False; First:=True; Last:=False;
    if SQL.Count>0 then
    begin
      if Pos('『今日+',SQL.text)>0 then DateMemoryValueSet(SQL.text);
      CNT:=0;
      for i:=0 to ARequest.ContentFields.Count-1 do
      begin
         if Pos('行番号:',ARequest.ContentFields.Names[i])=1 then CNT+=1;
         if UpperCase(ARequest.ContentFields.Names[i])='ファイル' then
         if ARequest.Files.Count>0 then
         begin
           UploadPath:=ARequest.ContentFields.Values['パス'];
           if Length(UploadPath)>0 then UploadPath:=StringReplace(UploadPath,'/',PathDelim,[rfReplaceAll])+PathDelim;
           UploadPath:= Folder+'images'+PathDelim+UploadPath; // /home/nocode/web/lib/i386-linux
           UploadFileName:=ARequest.Files[0].FileName;
           if Length(ARequest.ContentFields.Values['ファイル名変更'])>0 then
           if Length(ARequest.ContentFields.Values[ARequest.ContentFields.Values['ファイル名変更']])>0 then
           begin
             UploadFileName:=ARequest.ContentFields.Values[ARequest.ContentFields.Values['ファイル名変更']]+LowerCase(ExtractFileExt(UploadFileName));
           end;
           if ExtractFileNameWithoutExt(ARequest.Files[0].FileName)='削除' then
           begin
             if FileExists(UploadPath+UploadFileName) then DeleteFile(UploadPath+UploadFileName);
           end
           else begin
             f := ARequest.Files[0];
             try
               UploadFileStream :=TFilestream.Create(UploadPath+UploadFileName, fmCreate);   //fmOpenwrite
               try
                 UploadFileStream.position := UploadFileStream.size;
                 UploadFileStream.copyfrom(f.Stream,0);// appends
               finally
               end;
             finally
               UploadFileStream.free;
             end;
           end;
         end;
         if UpperCase(ARequest.ContentFields.Names[i])='CSVファイル' then
         begin
           Loop:=True;
           if ARequest.Files.Count>0 then
           begin
             f := ARequest.Files[0];
             try
               outstream :=TStringStream.Create;
               outstream.LoadFromStream(f.Stream);
               csv:=TStringlist.Create;
               csv.LoadFromStream(outstream);
               if csv.Count>0 then
               for k:=csv.Count-1 downto 0 do if csv[k]='' then csv.Delete(k);
               ARequest.ContentFields.Values['CSV列']:=csv.Text;
               csv.Free;
             finally
               outstream.free;
             end;
           end;
         end;
      end;
      for i:=0 to ARequest.ContentFields.Count-1 do
      begin
        if i=ARequest.ContentFields.Count-1 then Last:=True;
//log.Add(ARequest.ContentFields.Names[i]+'='+ARequest.ContentFields.ValueFromIndex[i]);
//log.Add(IntToStr(CNT));
//log.SaveToFile(Folder+'log.txt');
        if Pos('行番号:',ARequest.ContentFields.Names[i])=1 then
        begin
//log.Add(ARequest.ContentFields.Names[i]+'='+ARequest.ContentFields.ValueFromIndex[i]);
//log.Add(copy(ARequest.ContentFields.Names[i],4));
//log.SaveToFile(Folder+'log.txt');
          Loop:=True;
          CNT-=1;
          if CNT<=0 then Last:=True;
          if CNT>=0 then SQLexecute(StringReplace(ARequest.ContentFields.Names[i],'行番号','',[]),First,Last,False);
          First:=False;
          if Last=True then Break;
        end;
        if (Pos('TAB列',ARequest.ContentFields.Names[i])>0) or (Pos('CSV列',ARequest.ContentFields.Names[i])>0) then
        begin
          Loop:=True;
          Rows:=TStringlist.Create;
          csv:=TStringlist.Create;
          Rows.Text:=StringReplace(ARequest.ContentFields.ValueFromIndex[i],#13,'',[rfReplaceAll]);
          if Rows.Count>0 then
          for j:=Rows.Count-1 downto 0 do if Rows[j]='' then Rows.Delete(j);
          if Rows.Count>0 then
          for j:=0 to Rows.Count-1 do
          begin
            if Pos('CSV列',ARequest.ContentFields.Names[i])>0 then
            begin
              csv.Clear;
              csv.Delimiter:=',';
              csv.StrictDelimiter:=True;
              csv.DelimitedText:=Rows[j];
              if csv.Count>0 then
              for k:=csv.Count-1 downto 0 do if csv[k]='' then csv.Delete(k);
              Rows[j]:=StringReplace(StringReplace(csv.Text,#10,#9,[rfReplaceAll]),#13,'',[rfReplaceAll]); ;
            end;
//log.Add(IntToStr(j)+':'+Rows[j]);
//log.SaveToFile(Folder+'log.txt');
            if j = Rows.Count-1 then Last:=True;
            ARequest.ContentFields.Values['行番号']:=IntToStr(j+1);
            SQLexecute(Rows[j]+#9,First,Last,False);
          end;
//log.SaveToFile(Folder+'log.txt');
          csv.Free;
          Rows.Free;
        end;
      end;
      if Loop=False then SQLexecute('',True,True,False);
    end;
  end;

  SQL:=GetSQLetc('SQL',SQLname,'表示用のSQL文等',org); //'入力一件分のSQL文等' '入力開始時のSQL文等'
  if SQL.Count=0 then
  begin
    etc:=GetSQLetc('HTML','SQL不明','','');
    AResponse.Content := StringReplace(etc.Text,'＠SQL＠',SQLname,[]);
    SQL.Free;
    etc.Free;
    MenuNameList.Free;
    MenuDataList.Free;
    NameList1.Free;
    DataList1.Free;
    NameList2.Free;
    DataList2.Free;
    NameList3.Free;
    DataList3.Free;
    NameList4.Free;
    DataList4.Free;
    Exit;
  end;
//log.Add(SQL.Text);
//log.SaveToFile(Folder+'log.txt');

  if Pos('『今日+',SQL.text)>0 then DateMemoryValueSet(SQL.text);
  SQLexecute('',True,True,True);
//log.Add(NameList.Text);
//log.Add(DataList.Text);
//log.SaveToFile(Folder+'log.txt');

  etcName:=ARequest.ContentFields.Values['HTML'];
  if etcName='' then etcName:=ARequest.QueryFields.Values['HTML'];
  if etcName='' then etcName:='一覧表';
  etc:=GetSQLetc('HTML',etcName,'','');
  if etc.Count= 0 then etc:=GetSQLetc('HTML','一覧表','','');
//log.Add(etc.Text);
//log.SaveToFile(Folder+'log.txt');
  Li:= DataList1.Count -1;
  if DataList1.Count>0 then
  while DataList1[Li]='' do
  begin
    DataList1.Delete(Li);
    Li:=Li-1;
    if Li<=0 then break;
  end;
  if ARequest.QueryFields.Values['SQL']='HTML編集'
    then AResponse.Content :=InsetHTML(etc.Text)
    else if Pos('JSON',UpperCase(ARequest.QueryFields.Values['SQL']))<1
    then if Pos('AI2',UpperCase(ARequest.QueryFields.Values['HTML']))<1
    then AResponse.Content := StringReplace(StringReplace(StringReplace(StringReplace(StringReplace(StringReplace(StringReplace(StringReplace(StringReplace(InsetHTML(etc.Text),'＠データ＠',DBtoHTML(NameList1,DataList1),[]),'＠データ2＠',DBtoHTML(NameList2,DataList2),[]),'＠データ3＠',DBtoHTML(NameList3,DataList3),[]),'＠データ4＠',DBtoHTML(NameList4,DataList4),[]),'＠メニュー＠',DBtoHTML(MenuNameList,MenuDataList),[]),'＠所属＠',org,[rfReplaceAll]),'＠HTML＠',etcName,[rfReplaceAll]),'＠SQL＠',SQLname,[rfReplaceAll]),'＠Port＠',IntToStr(Application.Port),[rfReplaceAll])
    else begin
           S:= StringReplace(StringReplace(NameList1.Text,#13#10,#9,[rfReplaceAll]),#10,#9,[rfReplaceAll]);
           S:= '"'+StringReplace(StringReplace(copy(S,1,length(S)-1)+#10+StringReplace(StringReplace(DataList1.Text,#9#13#10,#10,[rfReplaceAll]),#9#10,#10,[rfReplaceAll]),#9,'","',[rfReplaceAll]),#10,'"'+#10+'"',[rfReplaceAll]);
           AResponse.Content := StringReplace(InsetHTML(etc.Text),'＠データ＠',copy(S,1,length(S)-2),[])
         end
    else AResponse.Content := StringReplace(StringReplace(StringReplace(StringReplace(StringReplace(StringReplace(StringReplace(StringReplace(StringReplace(InsetHTML(etc.Text),'＠データ＠',DBtoJSON(NameList1,DataList1),[]),'＠データ2＠',DBtoJSON(NameList2,DataList2),[]),'＠データ3＠',DBtoJSON(NameList3,DataList3),[]),'＠データ4＠',DBtoJSON(NameList4,DataList4),[]),'＠メニュー＠',DBtoJSON(MenuNameList,MenuDataList),[]),'＠所属＠',org,[rfReplaceAll]),'＠HTML＠',etcName,[rfReplaceAll]),'＠SQL＠',SQLname,[rfReplaceAll]),'＠Port＠',IntToStr(Application.Port),[rfReplaceAll]);

log.Add('==== END ==== '+FormatDateTime('yyyy-mm-dd hh:nn:ss zzz',now)+'ms IPAddress:'+ARequest.RemoteAddress+' User:'+GetBasicAuth(ARequest.Authorization,'User')+'('+org+') '+'HTML:'+etcName+#10);
log.SaveToFile(Folder+'log.txt');

  MenuNameList.Free;
  MenuDataList.Free;
  NameList1.Free;
  DataList1.Free;
  NameList2.Free;
  DataList2.Free;
  NameList3.Free;
  DataList3.Free;
  NameList4.Free;
  DataList4.Free;
  SQL.Free;
  etc.Free;
//  AResponse.Code := 200;
//  AResponse.SendContent;
end;
//===================================================================================================
//==「http://127.0.0.1(またはローカルIPアドレス)/?SQL=SQLパラメタ&HTML=HTMLパラメタ」CGI実行部終了 ==
//===================================================================================================

procedure Init(ARequest: TRequest; AResponse: TResponse); // 初回(SQLetc.dbがなくSQLetc_Init.dbしかない状態)にWeb管理者を登録する処理
var
  html:TStringList;
  User,Pass:string;
//  Conn: TSQLite3Connection;
//  Query: TSQLQuery;
  org:String;
begin
  User := ARequest.ContentFields.Values['管理者名'];
  Pass := ARequest.ContentFields.Values['パスワード'];
  if User = EmptyStr then
  with AResponse.Contents do
  begin
    Add('<!doctype html>');
    Add('<html lang="ja">');
    Add('  <head>');
    Add('    <meta charset="UTF-8">');
    Add('    <title>初期設定画面</title>');
    Add('  </head>');
    Add('  <body>');
    Add('<form action="' + ARequest.URI + '" method="POST">'{'" method="GET">'});
    Add('Webシステム管理者のお名前とパスワードを入力してください♪</label><br>');
    Add('<label for="管理者名">管理者名:</label><input type="text" name="管理者名" id="管理者名" /><br>');
    Add('<label for="パスワード">パスワード:</label><input type="password" name="パスワード" /><br>');
    Add('<input type="submit" value="登録" /><br>');
    Add('</form>');
    Add('なお，管理者名とパスワードは変更できません。<br>また，パスワードはデータベースの暗号化に利用します。<br>パスワードを忘れると，<br>「DB Browser for SQLite」でデータベースを開くことができません。<br>ご注意ください。');
    Add('  </body>');
    Add('</html>');
  end
  else
  begin
    if FileExists(Folder+'SQLetc_Init.db') then CopyFile(Folder+'SQLetc_Init.db',Folder+'SQLetc.db',false);
    Conn := TSQLite3Connection.Create(nil);
    with Conn do begin
//      Password:=Pass;
      SQLiteLibraryName:=SQLiteCipher;// '/usr/lib/i386-linux-gnu/libsqlcipher.so.0.8.6'{'/usr/lib/i386-linux-gnu/libsqlite3.so.0.8.6'};
      DatabaseName := Folder+'SQLetc.db'; {/home/nocode/web/lib/i386-linux}
      open;
      Connected := True;
      Transaction := TSQLTransaction.Create(Conn);
    end;

    Conn.Transaction.Active := True;
    Query := TSQLQuery.Create(nil);
    with Query do begin
      DataBase := Conn;
      // Insert the values into the database
      // We're using ParamByName which prevents SQL Injection
      // http://wiki.freepascal.org/Working_With_TSQLQuery#Parameters_in_TSQLQuery.SQL
      SQL.Text:='CREATE TABLE IF NOT EXISTS 利用者情報(名前 PRIMARY KEY, 暗号化パスワード, 所属)';
      try
        ExecSQL;
      except
      end;
      Conn.Transaction.Commit;
      SQL.Text:='INSERT INTO 利用者情報 (名前, 暗号化パスワード, 所属) values (:名前,:暗号化パスワード,:所属)';
      Params.ParamByName('名前').AsString := User;
      Params.ParamByName('暗号化パスワード').AsString := EncodeStringBase64(EncryptString(Pass,GeneratePhrase));
      Params.ParamByName('所属').AsString := 'Web管理者';
      try
        ExecSQL;
      except
      end;
      Conn.Transaction.Commit;
    end;

    Query.Close;
    Query.Free;
    Conn.Free;

    org:=BasicAuthorization(ARequest.Authorization);
    if org='null' then AResponse:= BasicAuthorizationView(AResponse)
    else
    begin
      html:=TStringList.Create;
      html.LoadFromFile(Folder+'menu'+PathDelim+'index.html');
      AResponse.Content := html.Text{'This is my precious content'};
      html.Free;
//      AResponse.Code := 200;
//      AResponse.SendContent;
    end;
  end;
end;

procedure MyShowRequestException(AResponse: TResponse; E : Exception; var Flag : boolean); // SQL文実行エラー等CGIリクエスト処理中に例外処理が呼び出された時の処理
begin
  with AResponse.Contents do
  begin
    Add('<!doctype html>');
    Add('<html lang="ja">');
    Add('  <head>');
    Add('    <meta charset="UTF-8">');
    Add('    <title>SQLite実行エラー</title>');
    Add('  </head>');
    Add('  <body>');
    Add('    <big><b>SQLite実行エラー</b></big><hr>');
    Add('    SQL名：'+SQLname+'<br>');
    Add('    SQL文：'+StringReplace(StringReplace(SQLbun,'＃##＃',#10,[rfIgnoreCase,rfReplaceAll]),#10,'<br>',[rfIgnoreCase,rfReplaceAll])+'<br>');
    Add('    エラー内容：<hr>'+E.Message+'<hr>');
    Add('  </body>');
    Add('</html>');
  end;
  AResponse.Code := 200;
  AResponse.SendContent;
end;

{//+++++++++++++++++++++++++++ 以下は参考処理 +++++++++++++++++++++++++++
function File2Base64(const FileName: String): String; // 下記ImageBase64処理で使用するファイルをBase64エンコードする関数
var
  FileStream: TFileStream;
  Str: String;
begin
  Result := '';
  if FileExists(FileName) then begin
    FileStream := TFileStream.Create(FileName,fmOpenRead);
    if FileStream.Size > 0 then
      begin
        SetLength(Str, FileStream.Size div SizeOf(Char));
        FileStream.ReadBuffer(Str[1], FileStream.Size div SizeOf(Char));
        Result := EncodeStringBase64(Str);
      end;
    FileStream.Free;
  end;
end;

procedure ImageBase64(ARequest: TRequest; AResponse: TResponse); // <IMG src="data:image/png;base64,～を指定してPNG画像を表示する処理見本
var
  MyImagePathAndName:String;
  org:String;
begin
  org:=BasicAuthorization(ARequest.Authorization);
  if org='null' then AResponse:= BasicAuthorizationView(AResponse)
  else
  begin
  MyImagePathAndName:=Folder+'images/test.jpg';
  AResponse.Content :=
    '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">'#10+
    '<!doctype html>'#10+
    '<html lang="ja">'#10+
    '  <head>'#10+
    '    <meta charset="UTF-8">'#10+
    '    <title>MainPage</title>'#10+
    '    <meta http-equiv="content-type" content="text/html; charset=UTF-8">'#10+
    '  </head>'#10+
    '  <body>'#10+
    '    <h1> MainPage </h1>'#10+
    '    Hello!'#10+
    '          <IMG src="data:image/png;base64,'+File2Base64(MyImagePathAndName)+'" alt="Graphe" width="600" height="400" hspace="20" vspace="40" /IMG >'+
    '  </body>'#10+
    '</html>';
  end;
//  AResponse.Code := 200;
//  AResponse.SendContent;
end;

procedure Png(ARequest: TRequest; AResponse: TResponse); // <img src=～指定ではなく，PNG画像を編集して表示する処理見本
var
  png: TPortableNetworkGraphic;
begin
  png := TPortableNetworkGraphic.Create;
  png.LoadFromFile('./images/test.png'); // /home/pi/web/lib/i386-linux
//  try
//    png.SetSize(100, 100);
//    png.Canvas.TextOut(10, 10, 'Hello world!');

    AResponse.ContentType:='image/png';
    AResponse.ContentStream := TMemoryStream.Create;
    png.SaveToStream(AResponse.ContentStream);
    AResponse.ContentLength := AResponse.ContentStream.Size;
    AResponse.SendContent;
    AResponse.ContentStream := nil;
//  finally
    png.Free;
//  end;
end;

procedure Simple(ARequest: TRequest; AResponse: TResponse); // 画像付きのHTMLタグを編集し，ブラウザに表示する処理見本
var
 HTML : String;
 org:String;
begin
  org:=BasicAuthorization(ARequest.Authorization);
  if org='null' then AResponse:= BasicAuthorizationView(AResponse)
  else
  begin
  HTML := '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">'#10+
  '<!doctype html>'#10+
  '<html lang="ja">'#10+
  '  <head>'#10+
  '    <meta charset="UTF-8">'#10+
  '    <title>Hello World</title>'#10+
          '   </head>'#10+
          '   <body>'#10+
          '    <img src="images/Google.jpg">'#10+
          '   </body>'#10+
          '  </html>';
  AResponse.Content := HTML;
//  AResponse.Code := 200;
//  AResponse.SendContent;
  end;
end;
//+++++++++++++++++++++++++++ 以上は参考処理 +++++++++++++++++++++++++++}

procedure Upload(ARequest: TRequest; AResponse: TResponse); // ブラウザからファイルをアップロードする処理見本
var
  n: Integer;
  f: TUploadedFile;
  OutStream:TFilestream;
  Path,FileName:String;
  org:String;
begin
  org:=BasicAuthorization(ARequest.Authorization);
  if org='null' then AResponse:= BasicAuthorizationView(AResponse)
  else
  begin
  n := ARequest.Files.Count;
  if n = 0 then
    with AResponse.Contents do
    begin
      Add('<!doctype html>');
      Add('<html lang="ja">');
      Add('  <head>');
      Add('    <meta charset="UTF-8">');
      Add('    <title>Example</title>');
      Add('  </head>');
      Add('  <body>');
      Add('<form id="form" action="' + ARequest.URI + '" method="POST" enctype="multipart/form-data">');
      Add('<label for="name">Drag n drop or click to add file:</label>');
      Add('<input type="file" name="input" />');
      Add('<input type="submit" value="Send" />');
      Add('</form>');
      Add('  </body>');
      Add('</html>');
    end
  else
  begin
    f := ARequest.Files[0];
    Path:= Folder+'images'+PathDelim; // /home/nocode/web/lib/i386-linux
    FileName:= ARequest.Files[0].FileName;
    try
      outstream :=TFilestream.Create(Path+FileName, fmCreate);   //fmOpenwrite
      try
        outstream.position := Outstream.size;
        outstream.copyfrom(f.Stream,0);// appends
      finally
      end;
    finally
      outstream.free;
    end;

    AResponse.ContentStream :=  TFileStream.Create(Path+FileName, fmOpenRead);
     try
        AResponse.ContentType := 'application/octet-stream';
        AResponse.CustomHeaders.Values['Content-Disposition'] := 'attachment; filename=' + FileName;
        AResponse.ContentLength := AResponse.ContentStream.Size;
        AResponse.SendContent;
     finally
         AResponse.ContentStream.Free;
     end;
  end;
  end;
end;

//**********************************************************
//**            楽々Webシステム メインの実行部            **
//**********************************************************

{$R *.res}

begin
  Application.OnShowRequestException:=@MyShowRequestException;              // エラーが発生した時にシステムの自動例外処理ではなく指定した処理を実行する
  Folder:=ExtractFilePath(ParamStr(0));                                     // このアプリの所在するパスを取得する
  SQLite3:='';
  {$IFDEF WINDOWS}
  SQLiteCipher:=GetFilePath(Folder, 'sqlcipher.dll', true);                 // Windows版SQLite3暗号化ライブラリーがあればセット，無ければ空白
  {$ELSE}
  {$IFDEF CPUARM}
  SQLiteCipher:=GetFilePath('/usr/lib', 'libsqlcipher.so.*', true);         // Linux版SQLite3(32bit)暗号化ライブラリーがあればセット，無ければ空白
  SQLite3:=GetFilePath('/usr/lib', 'libsqlite3.so.*', true);                // Linux版SQLite3(32bit)通常ライブラリーがあればセット，無ければ空白
  {$ELSE}
  {$IFDEF CPUAARCH64}
  SQLiteCipher:=GetFilePath('/usr/lib/aarch64-linux-gnu', 'libsqlcipher.so.*', true);         // Linux版SQLite3(32bit)暗号化ライブラリーがあればセット，無ければ空白
  SQLite3:=GetFilePath('/usr/lib/aarch64-linux-gnu', 'libsqlite3.so.*', true);                // Linux版SQLite3(32bit)通常ライブラリーがあればセット，無ければ空白
  {$ELSE}
  SQLiteCipher:=GetFilePath('/usr/lib/i386-linux-gnu', 'libsqlcipher.so.*', true);         // Linux版SQLite3(32bit)暗号化ライブラリーがあればセット，無ければ空白
  SQLite3:=GetFilePath('/usr/lib/i386-linux-gnu', 'libsqlite3.so.*', true);                // Linux版SQLite3(32bit)通常ライブラリーがあればセット，無ければ空白
  if SQLiteCipher='' then                                 // SQLite3暗号化ライブラリーがあればTrue，無ければFalseをセット
  begin
  SQLiteCipher:=GetFilePath('/usr/lib', 'libsqlcipher.so.*', true);         // Linux版SQLite3(32bit)暗号化ライブラリーがあればセット，無ければ空白
  SQLite3:=GetFilePath('/usr/lib', 'libsqlite3.so.*', true);                // Linux版SQLite3(32bit)通常ライブラリーがあればセット，無ければ空白
  end;
  {$ENDIF}
  {$ENDIF}
  {$ENDIF}
  if SQLiteCipher<>'' then CipherMode:=True                                 // SQLite3暗号化ライブラリーがあればTrue，無ければFalseをセット
  else begin
    CipherMode:=false;
    {$IFDEF WINDOWS}
    SQLiteCipher:=GetFilePath(Folder, 'sqlite3.dll', true);                 // Windows版SQLite3通常ライブラリーがあればセット，無ければ空白
    {$ELSE}
    {$IFDEF CPUARM}
    SQLiteCipher:=GetFilePath('/usr/lib', 'libsqlite.so.*', true);         // Linux版SQLite3(32bit)暗号化ライブラリーがあればセット，無ければ空白
    {$ELSE}
    {$IFDEF CPUAARCH64}
    SQLiteCipher:=GetFilePath('/usr/lib/aarch64-linux-gnu', 'libsqlite3.so.*', true);                // Linux版SQLite3(32bit)通常ライブラリーがあればセット，無ければ空白
    {$ELSE}
    SQLiteCipher:=GetFilePath('/usr/lib/i386-linux-gnu', 'libsqlite3.so.*', true);         // Linux版SQLite3通常ライブラリーがあればセット，無ければ空白
    if SQLiteCipher='' then SQLiteCipher:=GetFilePath('/usr/lib', 'libsqlite3.so.*', true);
    {$ENDIF}
    {$ENDIF}
    {$ENDIF}
  end;
  log:=TStringList.Create;
  if FileExists(Folder+'log.txt') then log.LoadFromFile(Folder+'log.txt');  // ログリスト準備
  Memory:=TStringList.Create;                                               // 置換文字『』用データリスト準備
  IPAddess:=GetIpAddrList;                                                  // 本マシンのローカルIPアドレスセット
  if FileExists(Folder+'SQLetc.db')=false then
  begin
    HTTPRouter.RegisterRoute('/',@Init, true);                              // SQLetc.dbが本アプリの所在するフォルダ上にない場合は，Init処理を実行
  end
  else begin
    {$IFDEF WINDOWS}
    RegisterFileLocation('images','images\');                               // SQLetc.dbが本アプリの所在するフォルダ上にある場合は，http://IPアドレス/images/～でファイル呼び出しを可能にする
    {$ELSE}
    RegisterFileLocation('images',Folder+'images/');
    {$ENDIF}
    HTTPRouter.registerRoute('/', @CGI, true);                              // SQLetc.dbが本アプリの所在するフォルダ上にある場合は，CGI処理を実行
    HTTPRouter.registerRoute('/upload', @Upload);     // ブラウザからファイルをアップロードする処理見本を使用する場合
{
    HTTPRouter.registerRoute('/png', @Png);           // <img src=～指定ではなく，PNG画像を編集して表示する処理見本を使用する場合
    HTTPRouter.registerRoute('/img64', @ImageBase64); // <IMG src="data:image/png;base64,～を指定してPNG画像を表示する処理見本を使用する場合
    HTTPRouter.registerRoute('/simple', @Simple);     // 画像付きのHTMLタグを編集し，ブラウザに表示する処理見本を使用する場合
}
  end;

  Application.Title:='Raku2WebSystem';                // アプリ名をタイトルにセット
  {$IFDEF WINDOWS}                                    // 通常は80番ポート。ただし，Windows版の場合は，アプリ名内に全角：があれば，その後ろの数値をポート番号にセット(http://127.0.0.1:8080/～のようにURL指定)
  if Pos('：',Application.ExeName)=0 then Application.Port:=80
  else begin
    Application.Port:=StrToIntDef(copy(ChangeFileExt(Application.ExeName,''),Pos('：',Application.ExeName)+1,MaxInt),80);
  end;
  {$ELSE}                                             // 通常は80番ポート。ただし，Linux版の場合は，アプリ名内に半角:があれば，その後ろの数値をポート番号にセット(http://127.0.0.1:8080/～のようにURL指定)
  if Pos(':',Application.ExeName)=0 then Application.Port:=80
  else begin
    Application.Port:=StrToIntDef(copy(Application.ExeName,Pos(':',Application.ExeName)+1,MaxInt),80);
  end;
  {$ENDIF}
  Application.Initialize;                             // アプリ初期化
  Application.Run;                                    // アプリ実行

  Memory.Free;                                        // 実行終了時，置換文字『』用データリスト解放
  log.Free;                                           // 実行終了時，ログリスト解放
end.


